@@ -12,8 +12,6 @@
 #
 ##############################################################################
 """Indexes appropriate for zope.catalog
-
-$Id: catalogindex.py 2918 2005-07-19 22:12:38Z jim $
 """
 import zope.interface
 
@@ -24,17 +22,18 @@ import zope.index.interfaces
 import zc.catalog.index
 import zc.catalog.interfaces
 
+
+@zope.interface.implementer(zc.catalog.interfaces.ICatalogValueIndex)
 class ValueIndex(zope.catalog.attribute.AttributeIndex,
                  zc.catalog.index.ValueIndex,
                  zope.container.contained.Contained):
+    pass
 
-    zope.interface.implements(zc.catalog.interfaces.ICatalogValueIndex)
-
+@zope.interface.implementer(zc.catalog.interfaces.ICatalogSetIndex)
 class SetIndex(zope.catalog.attribute.AttributeIndex,
                zc.catalog.index.SetIndex,
                zope.container.contained.Contained):
-
-    zope.interface.implements(zc.catalog.interfaces.ICatalogSetIndex)
+    pass
 
 class NormalizationWrapper(
     zope.catalog.attribute.AttributeIndex,
@@ -44,9 +43,10 @@ class NormalizationWrapper(
     pass
 
 
+@zope.interface.implementer(zc.catalog.interfaces.ICallableWrapper)
 class CallableWrapper(zc.catalog.index.CallableWrapper,
                       zope.container.contained.Contained):
-    zope.interface.implements(zc.catalog.interfaces.ICallableWrapper)
+    pass
 
 
 @zope.interface.implementer(
@@ -66,7 +66,7 @@ def DateTimeValueIndex(
     zope.interface.implementedBy(NormalizationWrapper),
     zc.catalog.interfaces.ISetIndex)
 def DateTimeSetIndex(
-    field_name=None, interface=None, field_callable=False, 
+    field_name=None, interface=None, field_callable=False,
     resolution=2): # hour; good for per-day searches
     ix = NormalizationWrapper(
         field_name, interface, field_callable, zc.catalog.index.SetIndex(),
