@@ -13,7 +13,6 @@
 ##############################################################################
 """extent catalog
 
-$Id: extentcatalog.py 3296 2005-09-09 19:29:20Z benji $
 """
 
 import sys
@@ -31,8 +30,9 @@ import zc.catalog
 from zc.catalog import interfaces
 
 
+@interface.implementer(interfaces.IExtent)
 class Extent(persistent.Persistent):
-    interface.implements(interfaces.IExtent)
+
     __parent__ = None
     family = BTrees.family32
 
@@ -95,6 +95,8 @@ class Extent(persistent.Persistent):
     def __nonzero__(self):
         return bool(self.set)
 
+    __bool__ = __nonzero__
+
     def __contains__(self, uid):
         return self.set.has_key(uid)
 
@@ -108,8 +110,8 @@ class Extent(persistent.Persistent):
             pass
 
 
+@interface.implementer(interfaces.IFilterExtent)
 class FilterExtent(Extent):
-    interface.implements(interfaces.IFilterExtent)
 
     def __init__(self, filter, family=None):
         super(FilterExtent, self).__init__(family=family)
@@ -125,6 +127,7 @@ class FilterExtent(Extent):
         return self.filter(self, uid, obj)
 
 
+@interface.implementer(interfaces.ISelfPopulatingExtent)
 class NonPopulatingExtent(Extent):
     """Base class for populating extent.
 
@@ -132,7 +135,6 @@ class NonPopulatingExtent(Extent):
     for catalogs that handle a very contained domain within an application.
     """
 
-    interface.implements(interfaces.ISelfPopulatingExtent)
 
     populated = False
 
@@ -140,8 +142,8 @@ class NonPopulatingExtent(Extent):
         self.populated = True
 
 
+@interface.implementer(interfaces.IExtentCatalog)
 class Catalog(catalog.Catalog):
-    interface.implements(interfaces.IExtentCatalog)
 
     UIDSource = None
 
@@ -156,7 +158,7 @@ class Catalog(catalog.Catalog):
 
         self.UIDSource = UIDSource
 
-        if extent.__parent__ is not None:
+        if extent.__parent__ is not None: # pragma: no cover
             raise ValueError("extent's __parent__ must be None")
         super(Catalog, self).__init__(family=extent.family)
         self.extent = extent
