@@ -13,7 +13,6 @@
 ##############################################################################
 """indexes, as might be found in zope.index
 
-$Id: index.py 2918 2005-07-19 22:12:38Z jim $
 """
 import sys
 import datetime
@@ -44,23 +43,34 @@ class FamilyProperty(object):
         d = instance.__dict__
         if "family" in d:
             return d["family"]
+
         if "btreemodule" in d:
             iftype = d["btreemodule"].split(".")[-1][:2]
             if iftype == "IF":
                 d["family"] = BTrees.family32
             elif iftype == "LF":
                 d["family"] = BTrees.family64
-            else:
+            else: # pragma: no cover
                 raise ValueError("can't determine btree family based on"
                                  " btreemodule of %r" % (iftype,))
         else:
             d["family"] = BTrees.family32
         self._clear_old_cruft(instance)
+
+        if isinstance(instance, persistent.Persistent):
+            # Mutating the dict directly is not guaranteed to
+            # register with the data manager.
+            instance._p_changed = True
+
         return d["family"]
 
     def __set__(self, instance, value):
         instance.__dict__["family"] = value
         self._clear_old_cruft(instance)
+        if isinstance(instance, persistent.Persistent):
+            # Mutating the dict directly is not guaranteed to
+            # register with the data manager.
+            instance._p_changed = True
 
     def _clear_old_cruft(self, instance):
         d = instance.__dict__
@@ -72,13 +82,15 @@ class FamilyProperty(object):
             del d["BTreeAPI"]
 
 
+
+@interface.implementer(
+    zope.index.interfaces.IInjection,
+    zope.index.interfaces.IIndexSearch,
+    zope.index.interfaces.IStatistics,
+    zc.catalog.interfaces.IIndexValues,
+)
 class AbstractIndex(persistent.Persistent):
 
-    interface.implements(zope.index.interfaces.IInjection,
-                         zope.index.interfaces.IIndexSearch,
-                         zope.index.interfaces.IStatistics,
-                         zc.catalog.interfaces.IIndexValues,
-                         )
 
     family = FamilyProperty()
 
@@ -133,27 +145,31 @@ class AbstractIndex(persistent.Persistent):
                 return iter(values.keys(min, max, excludemin, excludemax))
 
     def containsValue(self, value):
-        return bool(self.values_to_documents.has_key(value))
+        try:
+            return bool(self.values_to_documents.has_key(value))
+        except TypeError:
+            return False
 
     def ids(self):
         return self.documents_to_values.keys()
 
 def parseQuery(query):
-    if isinstance(query, dict):
-        if len(query) > 1:
-            raise ValueError(
-                'may only pass one of key, value pair')
-        elif not query:
-            return None, None
-        query_type, query = query.items()[0]
-        query_type = query_type.lower()
-    else:
+    if not isinstance(query, dict): # pragma: no cover
         raise ValueError('may only pass a dict to apply')
+
+    if len(query) > 1:
+        raise ValueError(
+            'may only pass one of key, value pair')
+    elif not query:
+        return None, None
+    query_type, query = list(query.items())[0]
+    query_type = query_type.lower()
     return query_type, query
 
+
+@interface.implementer(zc.catalog.interfaces.IValueIndex)
 class ValueIndex(SortingIndexMixin, AbstractIndex):
 
-    interface.implements(zc.catalog.interfaces.IValueIndex)
 
     # attributes used by sorting mixin
     _sorting_num_docs_attr = 'documentCount'        # Length object
@@ -206,9 +222,12 @@ class ValueIndex(SortingIndexMixin, AbstractIndex):
         if query_type is None:
             res = None
         elif query_type == 'any_of':
-            res = self.family.IF.multiunion(
-                [s for s in (values_to_documents.get(v) for v in query)
-                 if s is not None])
+            try:
+                res = self.family.IF.multiunion(
+                    [s for s in (values_to_documents.get(v) for v in query)
+                     if s is not None])
+            except TypeError:
+                return []
         elif query_type == 'any':
             if query is None:
                 res = self.family.IF.Set(self.ids())
@@ -216,10 +235,13 @@ class ValueIndex(SortingIndexMixin, AbstractIndex):
                 assert zc.catalog.interfaces.IExtent.providedBy(query)
                 res = query & self.family.IF.Set(self.ids())
         elif query_type == 'between':
-            res = self.family.IF.multiunion(
-                [s for s in (values_to_documents.get(v) for v in
-                             values_to_documents.keys(*query))
-                 if s is not None])
+            try:
+                res = self.family.IF.multiunion(
+                    [s for s in (values_to_documents.get(v) for v in
+                                 values_to_documents.keys(*query))
+                     if s is not None])
+            except TypeError:
+                return []
         elif query_type == 'none':
             assert zc.catalog.interfaces.IExtent.providedBy(query)
             res = query - self.family.IF.Set(self.ids())
@@ -244,9 +266,9 @@ class ValueIndex(SortingIndexMixin, AbstractIndex):
             else:
                 return (value,)
 
-class SetIndex(AbstractIndex):
 
-    interface.implements(zc.catalog.interfaces.ISetIndex)
+@interface.implementer(zc.catalog.interfaces.ISetIndex)
+class SetIndex(AbstractIndex):
 
     def _add_values(self, doc_id, added):
         values_to_documents = self.values_to_documents
@@ -305,8 +327,11 @@ class SetIndex(AbstractIndex):
         elif query_type == 'any_of':
             res = self.family.IF.Bucket()
             for v in query:
-                _, res = self.family.IF.weightedUnion(
-                    res, values_to_documents.get(v))
+                try:
+                    _, res = self.family.IF.weightedUnion(
+                        res, values_to_documents.get(v))
+                except TypeError:
+                    continue
         elif query_type == 'any':
             if query is None:
                 res = self.family.IF.Set(self.ids())
@@ -318,21 +343,30 @@ class SetIndex(AbstractIndex):
             values = iter(query)
             empty = self.family.IF.TreeSet()
             try:
-                res = values_to_documents.get(values.next(), empty)
+                res = values_to_documents.get(next(values), empty)
             except StopIteration:
                 res = empty
+            except TypeError:
+                return []
+
             while res:
                 try:
-                    v = values.next()
+                    v = next(values)
                 except StopIteration:
                     break
-                res = self.family.IF.intersection(
-                    res, values_to_documents.get(v, empty))
+                try:
+                    res = self.family.IF.intersection(
+                        res, values_to_documents.get(v, empty))
+                except TypeError:
+                    return []
         elif query_type == 'between':
             res = self.family.IF.Bucket()
-            for v in values_to_documents.keys(*query):
-                _, res = self.family.IF.weightedUnion(
-                    res, values_to_documents.get(v))
+            try:
+                for v in values_to_documents.keys(*query):
+                    _, res = self.family.IF.weightedUnion(
+                        res, values_to_documents.get(v))
+            except TypeError:
+                return []
         elif query_type == 'none':
             assert zc.catalog.interfaces.IExtent.providedBy(query)
             res = query - self.family.IF.Set(self.ids())
@@ -341,9 +375,9 @@ class SetIndex(AbstractIndex):
                 "unknown query type", query_type)
         return res
 
-class NormalizationWrapper(persistent.Persistent):
 
-    interface.implements(zc.catalog.interfaces.INormalizationWrapper)
+@interface.implementer(zc.catalog.interfaces.INormalizationWrapper)
+class NormalizationWrapper(persistent.Persistent):
 
     index = normalizer = None
     collection_index = False
@@ -427,9 +461,9 @@ class NormalizationWrapper(persistent.Persistent):
         # delegate upstream or raise AttributeError
         return self.index.sort
 
-class CallableWrapper(persistent.Persistent):
 
-    interface.implements(zc.catalog.interfaces.ICallableWrapper)
+@interface.implementer(zc.catalog.interfaces.ICallableWrapper)
+class CallableWrapper(persistent.Persistent):
 
     converter = None
     index = None
@@ -490,9 +524,9 @@ def day_begin(value):
         value, datetime.time(tzinfo=get_tz()))
 
 
+@interface.implementer(zc.catalog.interfaces.IDateTimeNormalizer)
 class DateTimeNormalizer(persistent.Persistent):
 
-    interface.implements(zc.catalog.interfaces.IDateTimeNormalizer)
     def __init__(self, resolution=2):
         self.resolution = resolution
         # 0, 1, 2, 3, 4
@@ -547,5 +581,5 @@ def DateTimeValueIndex(resolution=2): # 2 == minute; note that hour is good
 def DateTimeSetIndex(resolution=2): # 2 == minute; note that hour is good
     # for timezone-aware per-day searches
     ix = NormalizationWrapper(SetIndex(), DateTimeNormalizer(resolution), True)
-    interface.alsoProvides(ix, zc.catalog.interfaces.ISetIndex)    
+    interface.alsoProvides(ix, zc.catalog.interfaces.ISetIndex)
     return ix
