@@ -13,13 +13,12 @@
 ##############################################################################
 """A stemmer based on the textindexng stemmer, itself based on snowball.
 
-$Id: stemmer.py 2918 2005-07-19 22:12:38Z jim $
 """
 import re
 broken = None
 try:
     from zopyx.txng3.ext import stemmer
-except ImportError:
+except ImportError: # pragma: no cover
     try:
         from zopyx.txng3 import stemmer
     except ImportError:
@@ -27,7 +26,7 @@ except ImportError:
             import txngstemmer as stemmer
         except ImportError:
             stemmer = None
-            class Broken:
+            class Broken(object):
                 def stem(self, l):
                     return l
             broken = Broken()
@@ -57,8 +56,8 @@ class Stemmer(object):
         result = []
         for s in lst:
             try:
-                s = unicode(s)
-            except UnicodeDecodeError:
+                s = s.decode('utf-8') if isinstance(s, bytes) else s
+            except UnicodeDecodeError: # pragma: no cover
                 pass
             else:
                 s = stemmer.stem((s,))[0]
@@ -72,8 +71,8 @@ class Stemmer(object):
         for s in lst:
             if not rxGlob.search(s):
                 try:
-                    s = unicode(s)
-                except UnicodeDecodeError:
+                    s = s.decode('utf-8') if isinstance(s, bytes) else s
+                except UnicodeDecodeError: # pragma: no cover
                     pass
                 else:
                     s = stemmer.stem((s,))[0]
